package com.snowm.scanner.services

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.location.Location
import android.os.Bundle
import com.google.android.gms.location.*
import com.google.gson.Gson
import com.snowm.scanner.helpers.SnowMBackgroundScanner.Companion.startBackgroundScanning
import com.snowm.scanner.helpers.SnowMBackgroundScanner.Companion.stopBackgroundScanning


open class SnowMGeofenceReceiver : BroadcastReceiver() {
    lateinit var context: Context

    private var sharedPreferences: SharedPreferences? = null
    private var locationProviderClient: FusedLocationProviderClient? = null

    override fun onReceive(context: Context, intent: Intent) {
        this.context = context
        sharedPreferences = context.getSharedPreferences("snowm_scanner", Context.MODE_PRIVATE)
        locationProviderClient = LocationServices.getFusedLocationProviderClient(context)
        if (intent.action == "android.intent.action.BOOT_COMPLETED") {
            checkLocationAndStartScanning()
        } else if (intent.action == "io.snowm.geofence") {
            val geofencingEvent = GeofencingEvent.fromIntent(intent)
            if (geofencingEvent.hasError()) {
                val errorMessage: String = GeofenceStatusCodes.getStatusCodeString(geofencingEvent.errorCode)
                println(errorMessage)
                return
            }
            geofencingEvent.triggeringGeofences.forEach {
                if (it.requestId.contains("backgroundGeofence")) {
                    checkLocationAndStartScanning()
                    return@forEach
                }
                if (geofencingEvent.geofenceTransition == Geofence.GEOFENCE_TRANSITION_ENTER) {
                    didEnterGeofence(it)
                } else if (geofencingEvent.geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {
                    didExitGeofence(it)
                }
            }
        }
    }


    private fun checkLocationAndStartScanning() {
        val scannerDataString = sharedPreferences?.getString("backgroundScanner", null)
        if (scannerDataString != null) {
            locationProviderClient?.lastLocation?.addOnCompleteListener { location ->
                val bundle: Bundle = Gson().fromJson(scannerDataString, Bundle::class.java) as Bundle
                val geofences = bundle.getSerializable("geofences") as ArrayList<Map<String, Any>>
                if (geofences.count() == 0) {
                    startBackgroundScanning(context, bundle)
                    return@addOnCompleteListener
                }

                geofences.forEach {
                    val centerLatitude = it["latitude"] as Double
                    val centerLongitude = it["longitude"] as Double
                    val radius = it["longitude"] as Double
                    if (!isWithinGeofence(centerLatitude, centerLongitude, location.result?.latitude!!, location.result?.longitude!!, radius.toInt())) {
                        startBackgroundScanning(context, bundle)
                        return@addOnCompleteListener
                    }
                }
                stopBackgroundScanning(context)
            }
        }
    }

    private fun isWithinGeofence(centerLatitude: Double, centerLongitude: Double, testLatitude: Double, testLongitude: Double, radius: Int): Boolean {
        val results = FloatArray(1)
        Location.distanceBetween(centerLatitude, centerLongitude, testLatitude, testLongitude, results)
        val distanceInMeters = results[0]
        return distanceInMeters < radius
    }

    open fun didEnterGeofence(geofence: Geofence) {}

    open fun didExitGeofence(geofence: Geofence) {}

}